@@ -22,7 +22,7 @@ class StorageAccountAzureServicesAccessEnabled(BaseResourceCheck):
                 return CheckResult.UNKNOWN
 
             nacls = properties.get("networkAcls")
-            if nacls:
+            if nacls and isinstance(nacls, dict):
                 default_action = nacls.get("defaultAction")
                 if default_action == "Deny":
                     bypass = nacls.get("bypass")
