@@ -1,4 +1,4 @@
-from typing import Any
+from typing import Any, Dict
 
 
 class Edge:
@@ -20,3 +20,10 @@ class Edge:
 
     def __hash__(self) -> int:
         return hash(str(self))
+
+    def to_dict(self) -> Dict[str, Any]:
+        return {
+            'origin': self.origin,
+            'dest': self.dest,
+            'label': self.label
+        }
