@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import copy
 import io
 import logging
 import multiprocessing
@@ -14,18 +13,25 @@ import tempfile
 import yaml
 from typing import Optional, Dict, Any, TextIO, TYPE_CHECKING
 
+from checkov.common.parallelizer.parallel_runner import parallel_runner
+
+
 from checkov.common.graph.graph_builder import CustomAttributes
 from checkov.common.graph.graph_builder.consts import GraphSource
-from checkov.common.images.image_referencer import fix_related_resource_ids
+from checkov.common.images.image_referencer import Image
 from checkov.common.output.record import Record
 from checkov.common.output.report import Report
 from checkov.common.bridgecrew.check_type import CheckType
 from checkov.common.runners.base_runner import BaseRunner, filter_ignored_paths
-from checkov.common.typing import _CheckResult
-from checkov.kubernetes.kubernetes_utils import get_resource_id
-from checkov.kubernetes.runner import Runner as K8sRunner
-from checkov.kubernetes.runner import _get_entity_abs_path
-from checkov.kustomize.utils import get_kustomize_version
+from checkov.common.typing import _CheckResult, _EntityContext
+from checkov.common.util.consts import START_LINE, END_LINE
+from checkov.common.util.data_structures_utils import pickle_deepcopy
+from checkov.common.util.type_forcers import convert_str_to_bool
+from checkov.kubernetes.kubernetes_utils import create_check_result, get_resource_id, calculate_code_lines, \
+    PARENT_RESOURCE_ID_KEY_NAME
+from checkov.kubernetes.runner import Runner as K8sRunner, _get_entity_abs_path, _KubernetesContext, _KubernetesDefinitions
+from checkov.kustomize.image_referencer.manager import KustomizeImageReferencerManager
+from checkov.kustomize.utils import get_kustomize_version, get_kubectl_version
 from checkov.runner_filter import RunnerFilter
 from checkov.common.graph.checks_infra.registry import BaseRegistry
 from checkov.common.typing import LibraryGraphConnector
@@ -35,9 +41,12 @@ if TYPE_CHECKING:
     from checkov.common.checks.base_check import BaseCheck
     from checkov.common.graph.checks_infra.base_check import BaseGraphCheck
     from checkov.kubernetes.graph_manager import KubernetesGraphManager
+    from networkx import DiGraph
 
 
 class K8sKustomizeRunner(K8sRunner):
+    check_type = CheckType.KUSTOMIZE  # noqa: CCE003  # a static attribute
+
     def __init__(
         self,
         graph_class: type[KubernetesLocalGraph] = KubernetesLocalGraph,
@@ -48,33 +57,17 @@ class K8sKustomizeRunner(K8sRunner):
     ) -> None:
 
         super().__init__(graph_class, db_connector, source, graph_manager, external_registries, CheckType.KUSTOMIZE)
-        self.check_type = CheckType.KUSTOMIZE
         self.report_mutator_data: "dict[str, dict[str, Any]]" = {}
         self.original_root_dir: str = ''
         self.pbar.turn_off_progress_bar()
 
-    def run(
-        self,
-        root_folder: str | None,
-        external_checks_dir: list[str] | None = None,
-        files: list[str] | None = None,
-        runner_filter: RunnerFilter | None = None,
-        collect_skip_comments: bool = True
-    ) -> Report | list[Report]:
-        results = super().run(root_folder, external_checks_dir=external_checks_dir, runner_filter=runner_filter)
-
-        sca_image_report = None
-        if isinstance(results, list):
-            sca_image_report = next(result for result in results if result.check_type == CheckType.SCA_IMAGE)
-
-        if root_folder is not None:
-            fix_related_resource_ids(report=sca_image_report, tmp_dir=root_folder)
-
-        return results
+        # Allows using kustomize commands to directly edit the user's kustomization.yaml configurations
+        self.checkov_allow_kustomize_file_edits = convert_str_to_bool(os.getenv("CHECKOV_ALLOW_KUSTOMIZE_FILE_EDITS",
+                                                                                False))
 
     def set_external_data(
         self,
-        definitions: dict[str, dict[str, Any] | list[dict[str, Any]]] | None,
+        definitions: _KubernetesDefinitions | None,
         context: dict[str, dict[str, Any]] | None,
         breadcrumbs: dict[str, dict[str, Any]] | None,
         report_mutator_data: dict[str, dict[str, Any]] | None = None,
@@ -86,6 +79,14 @@ class K8sKustomizeRunner(K8sRunner):
     def set_report_mutator_data(self, report_mutator_data: Optional[Dict[str, Dict[str, Any]]]) -> None:
         self.report_mutator_data = report_mutator_data or {}
 
+    @staticmethod
+    def get_kustomize_resource_id(realKustomizeEnvMetadata: dict[str, Any], resource_id: str) -> str:
+        if 'overlay' in realKustomizeEnvMetadata["type"]:
+            kustomizeResourceID = f'{realKustomizeEnvMetadata["type"]}:{str(realKustomizeEnvMetadata["overlay_name"])}:{resource_id}'
+        else:
+            kustomizeResourceID = f'{realKustomizeEnvMetadata["type"]}:{resource_id}'
+        return kustomizeResourceID
+
     def mutate_kubernetes_results(
         self,
         results: dict[BaseCheck, _CheckResult],
@@ -95,6 +96,7 @@ class K8sKustomizeRunner(K8sRunner):
         file_abs_path: str,
         entity_conf: dict[str, Any],
         variable_evaluations: dict[str, Any],
+        root_folder: str | None = None
     ) -> Report:
         # Moves report generation logic out of checkov.kubernetes.runner.run() def.
         # Allows us to overriding report file information for "child" frameworks such as Kustomize, Helm
@@ -119,13 +121,14 @@ class K8sKustomizeRunner(K8sRunner):
                 continue
 
             realKustomizeEnvMetadata = kustomize_metadata[0][kustomize_file_mappings[file_abs_path]]
-            if 'overlay' in realKustomizeEnvMetadata["type"]:
-                kustomizeResourceID = f'{realKustomizeEnvMetadata["type"]}:{str(realKustomizeEnvMetadata["overlay_name"])}:{resource_id}'
-            else:
-                kustomizeResourceID = f'{realKustomizeEnvMetadata["type"]}:{resource_id}'
+            kustomizeResourceID = self.get_kustomize_resource_id(realKustomizeEnvMetadata, resource_id)
 
             external_run_indicator = "Bc"
             file_path = realKustomizeEnvMetadata['filePath']
+
+            caller_file_path = None
+            caller_file_line_range = None
+
             # means this scan originated in the platform
             if type(self.graph_manager).__name__.startswith(external_run_indicator):
                 absolute_file_path = file_abs_path
@@ -134,15 +137,27 @@ class K8sKustomizeRunner(K8sRunner):
                 # Fix file path to repo relative path
                 if self.original_root_dir:
                     repo_dir = str(pathlib.Path(self.original_root_dir).resolve())
+
                     if realKustomizeEnvMetadata['filePath'].startswith(repo_dir):
                         file_path = realKustomizeEnvMetadata['filePath'][len(repo_dir):]
 
+            if self.checkov_allow_kustomize_file_edits:
+                caller_resource_id = resource_id
+                if PARENT_RESOURCE_ID_KEY_NAME in entity_conf:
+                    caller_resource_id = entity_conf[PARENT_RESOURCE_ID_KEY_NAME]
+                caller_file_line_range, caller_file_path = self._get_caller_file_info(entity_context, k8_file,
+                                                                                      k8_file_path,
+                                                                                      resource_id,
+                                                                                      caller_resource_id,
+                                                                                      root_folder)
             code_lines = entity_context.get("code_lines")
             file_line_range = self.line_range(code_lines)
+
             record = Record(
                 check_id=check.id, bc_check_id=check.bc_id, check_name=check.name,
-                check_result=check_result, code_block=code_lines, file_path=file_path,
-                file_line_range=file_line_range,
+                check_result=check_result, code_block=code_lines,
+                file_path=file_path, file_line_range=file_line_range,
+                caller_file_path=caller_file_path, caller_file_line_range=caller_file_line_range,
                 resource=kustomizeResourceID, evaluations=variable_evaluations,
                 check_class=check.__class__.__module__, file_abs_path=absolute_file_path, severity=check.severity)
             record.set_guideline(check.guideline)
@@ -150,6 +165,98 @@ class K8sKustomizeRunner(K8sRunner):
 
         return report
 
+    def _get_caller_file_info(self, entity_context: _EntityContext, k8_file: str, k8_file_path: str, resource_id: str,
+                              caller_resource_id: str, root_folder: str | None) -> tuple[tuple[int, int] | None, str | None]:
+        origin_relative_path = entity_context.get('origin_relative_path')
+        if origin_relative_path is None:
+            return None, None
+        k8s_file_dir = pathlib.Path(k8_file_path).parent
+        raw_file_path = k8s_file_dir / origin_relative_path
+        caller_file_path = self._get_caller_file_path(k8s_file_dir, origin_relative_path, raw_file_path)
+        if root_folder is None:
+            return None, caller_file_path
+        caller_file_line_range = self._get_caller_line_range(root_folder, k8_file, origin_relative_path,
+                                                             resource_id, caller_resource_id)
+        return caller_file_line_range, caller_file_path
+
+    @staticmethod
+    def _get_caller_file_path(k8s_file_dir: pathlib.Path, origin_relative_path: str, raw_file_path: pathlib.Path)\
+            -> str:
+        """
+        Creates the correct file path based on the collection of metadata locations we have.
+
+        Example for expected input:
+            - k8s_fil_dir - Path('/resources/image_referencer/overlays/prod')
+            - origin_relative_path - '../../base/deployment.yaml'
+            - raw_file_path - Path('/resources/image_referencer/overlays/prod/../../base/deployment.yaml')
+        """
+        amount_of_parents = str.count(origin_relative_path, '..')
+        directory_prefix_path = k8s_file_dir
+        if amount_of_parents == 0:
+            # In case we don't have any relative paths, we need to remove the first directory parent
+            # (as the first directory is the same one of the kustomization.yaml file)
+            directory_prefix_path = k8s_file_dir.parent
+        elif amount_of_parents and len(k8s_file_dir.parents) >= amount_of_parents:
+            directory_prefix_path = k8s_file_dir.parents[amount_of_parents - 1]
+
+        directory_prefix = str(directory_prefix_path)
+        resolved_path = str(raw_file_path.resolve())
+        # Make sure the resolved path starts with the root folder, as pathlib.Path.resolve() might change it
+        if directory_prefix in resolved_path and not resolved_path.startswith(directory_prefix):
+            resolved_path = K8sKustomizeRunner._remove_extra_path_parts(resolved_path, directory_prefix)
+
+        return resolved_path[len(str(directory_prefix)):]
+
+    @staticmethod
+    def _remove_extra_path_parts(resolved_path: str, prefix: str) -> str:
+        """
+        Some pathlib paths can "add" extra arguments at the beginning after running `Path.resolve()`.
+        For example, running `Path('/var/example.txt').resolve` might result in `/<not-existent-dir>/var/example.txt`.
+        The purpose of this function is to remove any unintentional additions like this one.
+        """
+        resolved_path_parts = resolved_path.split(prefix)
+        if len(resolved_path_parts) > 1:
+            resolved_path = f'{prefix}{"".join(resolved_path_parts[1:])}'
+        else:
+            resolved_path = f'{prefix}{"".join(resolved_path_parts)}'
+        return resolved_path
+
+    def _get_caller_line_range(self, root_folder: str, k8_file: str, origin_relative_path: str,
+                               resource_id: str, caller_resource_id: str) -> tuple[int, int] | None:
+        raw_caller_directory = (pathlib.Path(k8_file.lstrip(os.path.sep)).parent /
+                                pathlib.Path(origin_relative_path.lstrip(os.path.sep)).parent)
+        caller_directory = str(pathlib.Path(f'{os.path.sep}{raw_caller_directory}').resolve())
+        caller_directory = K8sKustomizeRunner._remove_extra_path_parts(caller_directory, root_folder)
+        file_ending = pathlib.Path(origin_relative_path).suffix
+        caller_file_path = f'{str(pathlib.Path(caller_directory) / caller_resource_id.replace(".", "-"))}{file_ending}'
+
+        if caller_file_path not in self.definitions:
+            return None
+
+        caller_resource = None
+        for resource in self.definitions[caller_file_path]:
+            _resource_id = get_resource_id(resource)
+            if _resource_id == resource_id:
+                caller_resource = resource
+                break
+
+        if caller_resource is None:
+            return None
+
+        if caller_file_path not in self.definitions_raw:
+            # As we cannot calculate better lines with the `calculate_code_lines` without the raw code,
+            # we can use the existing info in the resource
+            return caller_resource[START_LINE], caller_resource[END_LINE]
+
+        raw_caller_resource = self.definitions_raw[caller_file_path]
+
+        caller_raw_start_line = caller_resource[START_LINE]
+        caller_raw_end_line = min(caller_resource[END_LINE], len(raw_caller_resource))
+
+        _, caller_start_line, caller_end_line = calculate_code_lines(raw_caller_resource, caller_raw_start_line,
+                                                                     caller_raw_end_line)
+        return caller_start_line, caller_end_line
+
     def line_range(self, code_lines: list[tuple[int, str]]) -> list[int]:
         num_of_lines = len(code_lines)
         file_line_range = [0, 0]
@@ -191,16 +298,33 @@ class K8sKustomizeRunner(K8sRunner):
                 else:
                     logging.warning(f"couldn't find {entity_file_abs_path} path in kustomizeFileMappings")
                     continue
+
+                caller_file_path = None
+                caller_file_line_range = None
+                if self.checkov_allow_kustomize_file_edits:
+                    caller_resource_id = entity_id
+                    if PARENT_RESOURCE_ID_KEY_NAME in entity:
+                        caller_resource_id = entity[PARENT_RESOURCE_ID_KEY_NAME]
+                    caller_file_line_range, caller_file_path = self._get_caller_file_info(entity_context,
+                                                                                          entity_file_path,
+                                                                                          entity_file_path,
+                                                                                          entity_id,
+                                                                                          caller_resource_id,
+                                                                                          root_folder)
                 code_lines = entity_context["code_lines"]
                 file_line_range = self.line_range(code_lines)
 
+                clean_check_result = create_check_result(check_result=check_result, entity_context=entity_context, check_id=check.id)
+
                 record = Record(
                     check_id=check.id,
                     check_name=check.name,
-                    check_result=check_result,
+                    check_result=clean_check_result,
                     code_block=code_lines,
                     file_path=realKustomizeEnvMetadata['filePath'],
                     file_line_range=file_line_range,
+                    caller_file_path=caller_file_path,
+                    caller_file_line_range=caller_file_line_range,
                     resource=kustomizeResourceID,  # entity.get(CustomAttributes.ID),
                     evaluations={},
                     check_class=check.__class__.__module__,
@@ -212,8 +336,32 @@ class K8sKustomizeRunner(K8sRunner):
 
         return report
 
+    def get_image_report(self, root_folder: str | None, runner_filter: RunnerFilter) -> Report | None:
+        if not self.graph_manager:
+            return None
+        return self.check_container_image_references(
+            graph_connector=self.graph_manager.get_reader_endpoint(),
+            root_path=self.original_root_dir,
+            runner_filter=runner_filter,
+        )
 
-class Runner(BaseRunner["KubernetesGraphManager"]):
+    def extract_images(
+            self,
+            graph_connector: DiGraph | None = None,
+            definitions: None = None,
+            definitions_raw: dict[str, list[tuple[int, str]]] | None = None
+    ) -> list[Image]:
+        if not graph_connector:
+            # should not happen
+            return []
+
+        manager = KustomizeImageReferencerManager(graph_connector=graph_connector, report_mutator_data=self.report_mutator_data)
+        images = manager.extract_images_from_resources()
+
+        return images
+
+
+class Runner(BaseRunner[_KubernetesDefinitions, _KubernetesContext, "KubernetesGraphManager"]):
     kustomize_command = 'kustomize'  # noqa: CCE003  # a static attribute
     kubectl_command = 'kubectl'  # noqa: CCE003  # a static attribute
     check_type = CheckType.KUSTOMIZE  # noqa: CCE003  # a static attribute
@@ -229,6 +377,9 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
         self.templateRendererCommand: str | None = None
         self.target_folder_path = ''
 
+        self.checkov_allow_kustomize_file_edits = convert_str_to_bool(os.getenv("CHECKOV_ALLOW_KUSTOMIZE_FILE_EDITS",
+                                                                                False))
+
     def get_k8s_target_folder_path(self) -> str:
         return self.target_folder_path
 
@@ -236,7 +387,7 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
         return {'kustomizeMetadata': self.kustomizeProcessedFolderAndMeta,
                 'kustomizeFileMappings': self.kustomizeFileMappings}
 
-    def _parseKustomization(self, kustomize_dir: str) -> dict[str, str]:
+    def _parseKustomization(self, kustomize_dir: str) -> dict[str, Any]:
         # We may have multiple results for "kustomization.yaml" files. These could be:
         # - Base and Environment (overlay) DIR's for the same kustomize-powered deployment
         # - OR, Multiple different Kustomize-powered deployments
@@ -253,29 +404,46 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
         else:
             return {}
 
-        with open(kustomization_path, 'r') as kustomizationFile:
-            metadata = {}
+        with open(kustomization_path, 'r') as kustomization_file:
+            metadata: dict[str, Any] = {}
             try:
-                fileContent = yaml.safe_load(kustomizationFile)
+                file_content = yaml.safe_load(kustomization_file)
             except yaml.YAMLError:
                 logging.info(f"Failed to load Kustomize metadata from {kustomization_path}.", exc_info=True)
+                return {}
 
-            if 'resources' in fileContent:
-                logging.debug(f"Kustomization contains resources: section. Likley a base. {kustomization_path}")
-                metadata['type'] = "base"
+            if not isinstance(file_content, dict):
+                return {}
 
-            elif 'patchesStrategicMerge' in fileContent:
-                logging.debug(f"Kustomization contains patchesStrategicMerge: section. Likley an overlay/env. {kustomization_path}")
+            if 'resources' in file_content and file_content['resources'] is not None:
+                resources = file_content['resources']
+
+                # We can differentiate between "overlays" and "bases" based on if the `resources` refers to a directory,
+                # which represents an "overlay", or only files which represents a "base"
+                resources_representing_directories = [r for r in resources if pathlib.Path(r).suffix == '']
+                if resources_representing_directories:
+                    logging.debug(
+                        f"Kustomization contains resources: section with directories. Likely an overlay/env."
+                        f" {kustomization_path}")
+                    metadata['type'] = "overlay"
+                    metadata['referenced_bases'] = resources_representing_directories
+                else:
+                    logging.debug(f"Kustomization contains resources: section with only files (no dirs). Likely a base."
+                                  f" {kustomization_path}")
+                    metadata['type'] = "base"
+
+            elif 'patchesStrategicMerge' in file_content:
+                logging.debug(f"Kustomization contains patchesStrategicMerge: section. Likely an overlay/env. {kustomization_path}")
                 metadata['type'] = "overlay"
-                if 'bases' in fileContent:
-                    metadata['referenced_bases'] = fileContent['bases']
+                if 'bases' in file_content:
+                    metadata['referenced_bases'] = file_content['bases']
 
-            elif 'bases' in fileContent:
-                logging.debug(f"Kustomization contains bases: section. Likley an overlay/env. {kustomization_path}")
+            elif 'bases' in file_content:
+                logging.debug(f"Kustomization contains bases: section. Likely an overlay/env. {kustomization_path}")
                 metadata['type'] = "overlay"
-                metadata['referenced_bases'] = fileContent['bases']
+                metadata['referenced_bases'] = file_content['bases']
 
-            metadata['fileContent'] = fileContent
+            metadata['fileContent'] = file_content
             metadata['filePath'] = f"{kustomization_path}"
             if metadata.get('type') == "base":
                 self.potentialBases.append(metadata['filePath'])
@@ -286,27 +454,18 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
         return metadata
 
     def check_system_deps(self) -> str | None:
-        # Ensure local system dependancies are available and of the correct version.
+        # Ensure local system dependencies are available and of the correct version.
         # Returns framework names to skip if deps **fail** (ie, return None for a successful deps check).
-        logging.info(f"Checking necessary system dependancies for {self.check_type} checks.")
+        logging.info(f"Checking necessary system dependencies for {self.check_type} checks.")
 
         if shutil.which(self.kubectl_command) is not None:
-            try:
-                proc = subprocess.run([self.kubectl_command, 'version', '--client=true'], capture_output=True)  # nosec
-                version_output = proc.stdout.decode("utf-8")
-
-                if "Client Version:" in version_output:
-                    kubectl_version_major = version_output.split('\n')[0].split('Major:\"')[1].split('"')[0]
-                    kubectl_version_minor = version_output.split('\n')[0].split('Minor:\"')[1].split('"')[0]
-                    kubectl_version = float(f"{kubectl_version_major}.{kubectl_version_minor}")
-                    if kubectl_version >= 1.14:
-                        logging.info(f"Found working version of {self.check_type} dependancy {self.kubectl_command}: {kubectl_version}")
-                        self.templateRendererCommand = self.kubectl_command
-                        return None
-
-            except Exception:
-                logging.debug(f"An error occured testing the {self.kubectl_command} command:", exc_info=True)
-
+            kubectl_version = get_kubectl_version(kubectl_command=self.kubectl_command)
+            if kubectl_version and kubectl_version >= 1.14:
+                logging.info(f"Found working version of {self.check_type} dependency {self.kubectl_command}: {kubectl_version}")
+                self.templateRendererCommand = self.kubectl_command
+                return None
+            else:
+                return self.check_type
         elif shutil.which(self.kustomize_command) is not None:
             kustomize_version = get_kustomize_version(kustomize_command=self.kustomize_command)
             if kustomize_version:
@@ -321,30 +480,31 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
             logging.info(f"Could not find usable tools locally to process {self.check_type} checks. Framework will be disabled for this run.")
             return self.check_type
 
-        return None
-
-    def _handle_overlay_case(self, file_path: str) -> None:
+    def _handle_overlay_case(self, file_path: str,
+                             kustomizeProcessedFolderAndMeta: dict[str, dict[str, Any]] | None = None) -> None:
+        if kustomizeProcessedFolderAndMeta is None:
+            kustomizeProcessedFolderAndMeta = self.kustomizeProcessedFolderAndMeta
         for parent in pathlib.Path(file_path).parents:
             for potentialBase in self.potentialBases:
                 pathlib_base_object = pathlib.Path(potentialBase)
                 potential_base_path = pathlib_base_object.parents[1]
                 if parent == potential_base_path.resolve():
-                    self.kustomizeProcessedFolderAndMeta[file_path]['calculated_bases'] = str(pathlib_base_object.parent)
+                    kustomizeProcessedFolderAndMeta[file_path]['calculated_bases'] = str(pathlib_base_object.parent)
         try:
-            relativeToFullPath = f"{file_path}/{self.kustomizeProcessedFolderAndMeta[file_path]['referenced_bases'][0]}"
-            if pathlib.Path(self.kustomizeProcessedFolderAndMeta[file_path]['calculated_bases']) == pathlib.Path(relativeToFullPath).resolve():
-                self.kustomizeProcessedFolderAndMeta[file_path]['validated_base'] = str(pathlib.Path(self.kustomizeProcessedFolderAndMeta[file_path]['calculated_bases']))
-                checkov_kustomize_env_name_by_path = str(pathlib.Path(file_path).relative_to(pathlib.Path(self.kustomizeProcessedFolderAndMeta[file_path]['calculated_bases']).parent))
-                self.kustomizeProcessedFolderAndMeta[file_path]['overlay_name'] = checkov_kustomize_env_name_by_path
-                logging.debug(f"Overlay based on {self.kustomizeProcessedFolderAndMeta[file_path]['validated_base']}, naming overlay {checkov_kustomize_env_name_by_path} for Checkov Results.")
+            relativeToFullPath = f"{file_path}/{kustomizeProcessedFolderAndMeta[file_path]['referenced_bases'][0]}"
+            if pathlib.Path(kustomizeProcessedFolderAndMeta[file_path]['calculated_bases']) == pathlib.Path(relativeToFullPath).resolve():
+                kustomizeProcessedFolderAndMeta[file_path]['validated_base'] = str(pathlib.Path(kustomizeProcessedFolderAndMeta[file_path]['calculated_bases']))
+                checkov_kustomize_env_name_by_path = str(pathlib.Path(file_path).relative_to(pathlib.Path(kustomizeProcessedFolderAndMeta[file_path]['calculated_bases']).parent))
+                kustomizeProcessedFolderAndMeta[file_path]['overlay_name'] = checkov_kustomize_env_name_by_path
+                logging.debug(f"Overlay based on {kustomizeProcessedFolderAndMeta[file_path]['validated_base']}, naming overlay {checkov_kustomize_env_name_by_path} for Checkov Results.")
             else:
                 checkov_kustomize_env_name_by_path = pathlib.Path(file_path).stem
-                self.kustomizeProcessedFolderAndMeta[file_path]['overlay_name'] = checkov_kustomize_env_name_by_path
+                kustomizeProcessedFolderAndMeta[file_path]['overlay_name'] = checkov_kustomize_env_name_by_path
                 logging.debug(f"Could not confirm base dir for Kustomize overlay/env. Using {checkov_kustomize_env_name_by_path} for Checkov Results.")
 
         except KeyError:
             checkov_kustomize_env_name_by_path = pathlib.Path(file_path).stem
-            self.kustomizeProcessedFolderAndMeta[file_path]['overlay_name'] = checkov_kustomize_env_name_by_path
+            kustomizeProcessedFolderAndMeta[file_path]['overlay_name'] = checkov_kustomize_env_name_by_path
             logging.debug(f"Could not confirm base dir for Kustomize overlay/env. Using {checkov_kustomize_env_name_by_path} for Checkov Results.")
 
     @staticmethod
@@ -392,15 +552,32 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
             line_num += 1
         return cur_writer
 
-    @staticmethod
-    def _get_kubectl_output(filePath: str, template_renderer_command: str, source_type: str | None) -> bytes:
+    def _get_kubectl_output(self, filePath: str, template_renderer_command: str, source_type: str | None) -> bytes | None:
         # Template out the Kustomizations to Kubernetes YAML
         if template_renderer_command == "kubectl":
             template_render_command_options = "kustomize"
-        if template_renderer_command == "kustomize":
+        elif template_renderer_command == "kustomize":
             template_render_command_options = "build"
-        proc = subprocess.Popen([template_renderer_command, template_render_command_options, filePath], stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # nosec
+        else:
+            logging.error(f"Template renderer command has an invalid value: {template_renderer_command}")
+            return None
+
+        add_origin_annotations_return_code = None
+
+        if self.checkov_allow_kustomize_file_edits:
+            add_origin_annotations_command = 'kustomize edit add buildmetadata originAnnotations'
+            add_origin_annotations_return_code = subprocess.run(add_origin_annotations_command.split(' '),  # nosec
+                                                                cwd=filePath).returncode
+
+        full_command = f'{template_renderer_command} {template_render_command_options}'
+        proc = subprocess.Popen(full_command.split(' '), cwd=filePath, stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # nosec
         output, _ = proc.communicate()
+
+        if self.checkov_allow_kustomize_file_edits and add_origin_annotations_return_code == 0:
+            # If the return code is not 0, we didn't add the new buildmetadata field, so we shouldn't remove it
+            remove_origin_annotaions = 'kustomize edit remove buildmetadata originAnnotations'
+            subprocess.run(remove_origin_annotaions.split(' '), cwd=filePath)  # nosec
+
         logging.info(
             f"Ran kubectl to build Kustomize output. DIR: {filePath}. TYPE: {source_type}.")
         return output
@@ -416,7 +593,7 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
                 env_or_base_path_prefix = ""
             else:
                 base_path_parts = pathlib.Path(kustomize_processed_folder_and_meta[file_path]['calculated_bases']).parts
-                most_significant_base_path = f"/{base_path_parts[-3]}/{base_path_parts[-2]}/{base_path_parts[-1]}"
+                most_significant_base_path = f"/{base_path_parts[-3]}/{base_path_parts[-2]}"
                 env_or_base_path_prefix = f"{most_significant_base_path}/{kustomize_processed_folder_and_meta[file_path]['overlay_name']}"
 
         elif kustomize_processed_folder_and_meta[file_path].get('type') == "base":
@@ -427,16 +604,29 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
 
         return env_or_base_path_prefix
 
-    @staticmethod
+    def get_binary_output_from_directory(
+            self,
+            file_path: str,
+            template_renderer_command: str,
+    ) -> tuple[bytes, str] | tuple[None, None]:
+        kustomizeProcessedFolderAndMeta = {file_path: self._parseKustomization(file_path)}
+        if kustomizeProcessedFolderAndMeta[file_path].get('type') == 'overlay':
+            self._handle_overlay_case(file_path, kustomizeProcessedFolderAndMeta)
+        return self.get_binary_output(file_path, kustomizeProcessedFolderAndMeta, template_renderer_command)
+
     def get_binary_output(
+        self,
         file_path: str,
         kustomize_processed_folder_and_meta: dict[str, dict[str, Any]],
         template_renderer_command: str,
     ) -> tuple[bytes, str] | tuple[None, None]:
         source_type = kustomize_processed_folder_and_meta[file_path].get('type')
-        logging.debug(f"Kustomization at {file_path} likley a {source_type}")
+        logging.debug(f"Kustomization at {file_path} likely a {source_type}")
         try:
-            output = Runner._get_kubectl_output(file_path, template_renderer_command, source_type)
+            output = self._get_kubectl_output(file_path, template_renderer_command, source_type)
+            if output is None:
+                return None, None
+
             return output, file_path
         except Exception:
             logging.warning(f"Error building Kustomize output at dir: {file_path}.", exc_info=True)
@@ -464,15 +654,15 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
         if cur_writer:
             Runner._curWriterValidateStoreMapAndClose(cur_writer, file_path, shared_kustomize_file_mappings)
 
-    @staticmethod
     def _run_kustomize_parser(
+        self,
         file_path: str,
         shared_kustomize_file_mappings: dict[str, str],
         kustomize_processed_folder_and_meta: dict[str, dict[str, Any]],
         template_renderer_command: str,
         target_folder_path: str,
     ) -> None:
-        output, _ = Runner.get_binary_output(file_path, kustomize_processed_folder_and_meta, template_renderer_command)
+        output, _ = self.get_binary_output(file_path, kustomize_processed_folder_and_meta, template_renderer_command)
         if not output:
             return
         Runner._parse_output(output, file_path, kustomize_processed_folder_and_meta, target_folder_path, shared_kustomize_file_mappings)
@@ -499,7 +689,7 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
 
             shared_kustomize_file_mappings: dict[str, str] = {}
             for file_path in self.kustomizeProcessedFolderAndMeta:
-                Runner._run_kustomize_parser(
+                self._run_kustomize_parser(
                     file_path=file_path,
                     shared_kustomize_file_mappings=shared_kustomize_file_mappings,
                     kustomize_processed_folder_and_meta=self.kustomizeProcessedFolderAndMeta,
@@ -511,25 +701,20 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
 
         manager = multiprocessing.Manager()
         # make sure we have new dict
-        shared_kustomize_file_mappings = copy.copy(manager.dict())  # type:ignore[arg-type]  # works with DictProxy
+        shared_kustomize_file_mappings = pickle_deepcopy(manager.dict())  # type:ignore[arg-type]  # works with DictProxy
         shared_kustomize_file_mappings.clear()
-        jobs = []
-        for filePath in self.kustomizeProcessedFolderAndMeta:
-            p = multiprocessing.Process(
-                target=Runner._run_kustomize_parser,
-                args=(
-                    filePath,
-                    shared_kustomize_file_mappings,
-                    self.kustomizeProcessedFolderAndMeta,
-                    self.templateRendererCommand,
-                    self.target_folder_path
-                )
-            )
-            jobs.append(p)
-            p.start()
 
-        for proc in jobs:
-            proc.join()
+        items = [
+            (
+                filePath,
+                shared_kustomize_file_mappings,
+                self.kustomizeProcessedFolderAndMeta,
+                self.templateRendererCommand,
+                self.target_folder_path,
+            )
+            for filePath in self.kustomizeProcessedFolderAndMeta
+        ]
+        list(parallel_runner.run_function(self._run_kustomize_parser, items))
 
         self.kustomizeFileMappings = dict(shared_kustomize_file_mappings)
 
@@ -562,8 +747,9 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
                 k8s_runner.original_root_dir = root_folder
 
             # the returned report can be a list of reports, which also includes an SCA image report
-            report = k8s_runner.run(target_dir, external_checks_dir=None, runner_filter=runner_filter)
-            logging.debug(f"Sucessfully ran k8s scan on Kustomization templated files in tmp scan dir : {target_dir}")
+            report = k8s_runner.run(target_dir, external_checks_dir=external_checks_dir, runner_filter=runner_filter)
+            self.graph_manager = k8s_runner.graph_manager
+            logging.debug(f"Successfully ran k8s scan on Kustomization templated files in tmp scan dir : {target_dir}")
 
             shutil.rmtree(target_dir)
 
@@ -589,6 +775,7 @@ class Runner(BaseRunner["KubernetesGraphManager"]):
             with open(currentFileName) as f:
                 currentYamlObject = yaml.safe_load(f)
                 # Validate we have a K8S manifest
+
                 if "apiVersion" in currentYamlObject:
                     itemName = []
                     itemName.append(currentYamlObject['kind'])
