@@ -4,7 +4,7 @@ from checkov.terraform.checks.resource.base_resource_check import BaseResourceCh
 
 class DropCapabilitiesPSP(BaseResourceCheck):
 
-    def __init__(self):
+    def __init__(self) -> None:
         # CIS-1.3 1.7.7
         # CIS-1.5 5.2.7
         name = "Do not admit containers with the NET_RAW capability"
@@ -17,11 +17,13 @@ class DropCapabilitiesPSP(BaseResourceCheck):
 
     def scan_resource_conf(self, conf) -> CheckResult:
         if conf.get('spec'):
+            self.evaluated_keys = ['spec']
             spec = conf.get('spec')[0]
             if not spec:
                 return CheckResult.UNKNOWN
 
             if spec.get("required_drop_capabilities"):
+                self.evaluated_keys = ['spec/[0]/required_drop_capabilities']
                 drop_cap = spec.get("required_drop_capabilities")[0]
                 if drop_cap and isinstance(drop_cap, list):
                     if any(cap in drop_cap for cap in ("ALL", "NET_RAW")):
