@@ -4,15 +4,15 @@ import logging
 import os
 import re
 from typing import Tuple
-from typing import Union, List, Any, Dict, Optional, Callable, TYPE_CHECKING
-
-import igraph
+from typing import Union, List, Any, Dict, Optional, TYPE_CHECKING
 
+from checkov.common.typing import LibraryGraph
 from checkov.common.util.parser_utils import TERRAFORM_NESTED_MODULE_PATH_SEPARATOR_LENGTH, \
     TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR
+from networkx import DiGraph
 
 if TYPE_CHECKING:
-    from networkx import DiGraph
+    from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock
 
 from checkov.common.util.type_forcers import force_int
 from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes
@@ -21,15 +21,15 @@ from checkov.terraform.graph_builder.variable_rendering.vertex_reference import
 
 MODULE_DEPENDENCY_PATTERN_IN_PATH = re.compile(r"\(\[\{.+\#\*\#.+\}\]\)")
 CHECKOV_RENDER_MAX_LEN = force_int(os.getenv("CHECKOV_RENDER_MAX_LEN", "10000"))
+CHECKOV_LOREM_IPSUM_VAL = '\x00'
 
 
 def is_local_path(root_dir: str, source: str) -> bool:
     # https://www.terraform.io/docs/modules/sources.html#local-paths
     return (
-        source.startswith("./")
-        or source.startswith("/./")
-        or source.startswith("../")
-        or source in os.listdir(root_dir)
+        source.startswith(r"./")
+        or source.startswith(r"/./")
+        or source.startswith(r"../")
     )
 
 
@@ -45,14 +45,14 @@ def remove_module_dependency_in_path(path: str) -> Tuple[str, str, str]:
     return path, module_and_num[0], module_and_num[1]
 
 
-def extract_module_dependency_path(module_dependency: List[str]) -> List[str]:
+def extract_module_dependency_path(module_dependency: str | List[str]) -> List[str]:
     """
     :param: module_dependency: a list looking like ['[path_to_module.tf#0]']
     :return: the path without enclosing array and index: 'path_to_module.tf'
     """
     if not module_dependency:
         return ["", ""]
-    if isinstance(module_dependency, list) and len(module_dependency) > 0:
+    if isinstance(module_dependency, list):
         module_dependency = module_dependency[0]
     return [
         module_dependency[3:module_dependency.index(f'.tf{TERRAFORM_NESTED_MODULE_INDEX_SEPARATOR}') + len('.tf')],
@@ -60,66 +60,92 @@ def extract_module_dependency_path(module_dependency: List[str]) -> List[str]:
     ]
 
 
-BLOCK_TYPES_STRINGS = ["var", "local", "module", "data"]
+BLOCK_TYPES_STRINGS = ("var", "local", "module", "data")
 FUNC_CALL_PREFIX_PATTERN = re.compile(r"([.a-zA-Z]+)\(")
-INTERPOLATION_PATTERN = re.compile(r"[${}]")
 INTERPOLATION_EXPR = re.compile(r"\$\{([^\}]*)\}")
 INDEX_PATTERN = re.compile(r"\[([0-9]+)\]")
 MAP_ATTRIBUTE_PATTERN = re.compile(r"\[\"([^\d\W]\w*)\"\]")
+NESTED_ATTRIBUTE_PATTERN = re.compile(r"\.\d+")
 
 
 def get_vertices_references(
-        str_value: str, aliases: Dict[str, Dict[str, BlockType]], resources_types: List[str]
+        str_value: str, aliases: Dict[str, Dict[str, str]], resources_types: List[str]
 ) -> List[TerraformVertexReference]:
-    vertices_references = []
+    has_interpolation = True if "${" in str_value else False
+    vertices_references: "list[TerraformVertexReference]" = []
     words_in_str_value = str_value.split()
 
     for word in words_in_str_value:
-        if word.startswith(".") or word.startswith("/."):
+        if word.startswith((".", r"/.")):
             # check if word is a relative path
             continue
 
-        interpolations = re.split(INTERPOLATION_EXPR, word)
-        for interpolation_content in interpolations:
-            for w in interpolation_content.split(","):
-                word_sub_parts = w.split(".")
-                if len(word_sub_parts) <= 1 or word_sub_parts[0].isnumeric():
-                    # if the word doesn't contain a '.' char, or if the first part before the dot is a number
-                    continue
-
-                suspected_block_type = word_sub_parts[0]
-                if suspected_block_type in BLOCK_TYPES_STRINGS:
-                    # matching cases like 'var.x'
-                    vertex_reference = TerraformVertexReference(
-                        block_type=suspected_block_type, sub_parts=word_sub_parts[1:], origin_value=w
-                    )
-                    if vertex_reference not in vertices_references:
-                        vertices_references.append(vertex_reference)
-                    continue
-
-                vertex_reference_alias = get_vertex_reference_from_alias(suspected_block_type, aliases, word_sub_parts)
-                if vertex_reference_alias and vertex_reference_alias not in vertices_references:
-                    vertex_reference_alias.origin_value = w
-                    # matching cases where the word is referring an alias
-                    vertices_references.append(vertex_reference_alias)
-                    continue
-
-                # matching cases like 'aws_vpc.main'
-                if word_sub_parts[0] in resources_types:
-                    block_name = word_sub_parts[0] + "." + word_sub_parts[1]
-                    word_sub_parts = [block_name] + word_sub_parts[2:]
-                    vertex_reference = TerraformVertexReference(
-                        block_type=BlockType.RESOURCE, sub_parts=word_sub_parts, origin_value=w
-                    )
-                    if vertex_reference not in vertices_references:
-                        vertices_references.append(vertex_reference)
+        if has_interpolation:
+            interpolations = re.split(INTERPOLATION_EXPR, word)
+            for interpolation_content in interpolations:
+                add_vertices_references_from_word(
+                    vertices_references=vertices_references,
+                    word=interpolation_content,
+                    aliases=aliases,
+                    resources_types=resources_types,
+                )
+        else:
+            add_vertices_references_from_word(
+                vertices_references=vertices_references,
+                word=word,
+                aliases=aliases,
+                resources_types=resources_types,
+            )
 
     return vertices_references
 
 
+def add_vertices_references_from_word(
+    vertices_references: list[TerraformVertexReference],
+    word: str,
+    aliases: dict[str, dict[str, str]],
+    resources_types: list[str],
+) -> None:
+    for w in word.split(","):
+        word_sub_parts = w.split(".")
+        if len(word_sub_parts) <= 1 or word_sub_parts[0].isnumeric():
+            # if the word doesn't contain a '.' char, or if the first part before the dot is a number
+            continue
+
+        suspected_block_type = word_sub_parts[0]
+        if suspected_block_type in BLOCK_TYPES_STRINGS:
+            # matching cases like 'var.x'
+            vertex_reference = TerraformVertexReference(
+                block_type=suspected_block_type, sub_parts=word_sub_parts[1:], origin_value=w
+            )
+            if vertex_reference not in vertices_references:
+                vertices_references.append(vertex_reference)
+            continue
+
+        vertex_reference_alias = get_vertex_reference_from_alias(suspected_block_type, aliases, word_sub_parts)
+        if vertex_reference_alias and vertex_reference_alias not in vertices_references:
+            vertex_reference_alias.origin_value = w
+            # matching cases where the word is referring an alias
+            vertices_references.append(vertex_reference_alias)
+            continue
+
+        # matching cases like 'aws_vpc.main'
+        if word_sub_parts[0] in resources_types:
+            block_name = word_sub_parts[0] + "." + word_sub_parts[1]
+            word_sub_parts = [block_name] + word_sub_parts[2:]
+            vertex_reference = TerraformVertexReference(
+                block_type=BlockType.RESOURCE, sub_parts=word_sub_parts, origin_value=w
+            )
+            if vertex_reference not in vertices_references:
+                vertices_references.append(vertex_reference)
+
+
 def get_vertex_reference_from_alias(
-        block_type_str: str, aliases: Dict[str, Dict[str, BlockType]], val: List[str]
+        block_type_str: str, aliases: Dict[str, Dict[str, str]], val: List[str]
 ) -> Optional[TerraformVertexReference]:
+    if not aliases:
+        return None
+
     block_type = ""
     if block_type_str in aliases:
         block_type = aliases[block_type_str][CustomAttributes.BLOCK_TYPE]
@@ -132,24 +158,45 @@ def get_vertex_reference_from_alias(
 
 
 def remove_function_calls_from_str(str_value: str) -> str:
+    if "(" not in str_value:
+        # otherwise it can't be a function call
+        return str_value
+
     # remove start of function calls:: 'length(aws_vpc.main) > 0 ? aws_vpc.main[0].cidr_block : ${var.x}' --> 'aws_vpc.main) > 0 ? aws_vpc.main[0].cidr_block : ${var.x}'
     str_value = re.sub(FUNC_CALL_PREFIX_PATTERN, "", str_value)
     # remove ')'
-    return re.sub(re.compile(r"[)]+"), "", str_value)
+    return str_value.replace(")", "")
 
 
 def remove_index_pattern_from_str(str_value: str) -> str:
+    if "[" not in str_value:
+        # otherwise it can't be accessed via index
+        return str_value
+
+    # Workaround for cases where the string value contains a map attribute access for foreach data block
+    # UT for this - `test_foreach_data_with_resource` -> /variable_rendering/test_foreach_renderer.py
+    if "data." in str_value:
+        return str_value
+
     str_value = re.sub(INDEX_PATTERN, "", str_value)
-    str_value = str_value.replace("[", " [ ")
-    str_value = str_value.replace("]", " ] ")
+    str_value = str_value.replace('["', CHECKOV_LOREM_IPSUM_VAL).replace("[", " [ ").replace(CHECKOV_LOREM_IPSUM_VAL, '["')
+    str_value = str_value.replace('"]', CHECKOV_LOREM_IPSUM_VAL).replace("]", " ] ").replace(CHECKOV_LOREM_IPSUM_VAL, '"]')
     return str_value
 
 
-def remove_interpolation(str_value: str, replace_str: str = " ") -> str:
-    return re.sub(INTERPOLATION_PATTERN, replace_str, str_value)
+def remove_interpolation(str_value: str) -> str:
+    if "${" not in str_value:
+        # otherwise it can't be a string interpolation
+        return str_value
+
+    return str_value.replace("${", " ").replace("}", " ")
 
 
 def replace_map_attribute_access_with_dot(str_value: str) -> str:
+    if "[\"" not in str_value:
+        # otherwise it can't be accessed via named index
+        return str_value
+
     split_by_identifiers = re.split(MAP_ATTRIBUTE_PATTERN, str_value)
     new_split = []
     for split_part in split_by_identifiers:
@@ -162,19 +209,10 @@ def replace_map_attribute_access_with_dot(str_value: str) -> str:
     return ".".join(new_split)
 
 
-DEFAULT_CLEANUP_FUNCTIONS: List[Callable[[str], str]] = [
-    remove_function_calls_from_str,
-    remove_index_pattern_from_str,
-    replace_map_attribute_access_with_dot,
-    remove_interpolation,
-]
-
-
 def get_referenced_vertices_in_value(
         value: Union[str, List[str], Dict[str, str]],
-        aliases: Dict[str, Dict[str, BlockType]],
+        aliases: Dict[str, Dict[str, str]],
         resources_types: List[str],
-        cleanup_functions: Optional[List[Callable[[str], str]]] = None,
 ) -> List[TerraformVertexReference]:
     references_vertices: "list[TerraformVertexReference]" = []
 
@@ -182,36 +220,52 @@ def get_referenced_vertices_in_value(
         # bool/int values can't have a references to other vertices
         return references_vertices
 
-    if cleanup_functions is None:
-        cleanup_functions = DEFAULT_CLEANUP_FUNCTIONS
-
     if isinstance(value, list):
         for sub_value in value:
             references_vertices += get_referenced_vertices_in_value(
-                sub_value, aliases, resources_types, cleanup_functions
+                sub_value, aliases, resources_types
             )
 
     if isinstance(value, dict):
         for sub_value in value.values():
             references_vertices += get_referenced_vertices_in_value(
-                sub_value, aliases, resources_types, cleanup_functions
+                sub_value, aliases, resources_types
             )
 
     if isinstance(value, str):
-        value_len = len(value)
-        if CHECKOV_RENDER_MAX_LEN and 0 < CHECKOV_RENDER_MAX_LEN < value_len:
-            logging.debug(f'Rendering was skipped for a {value_len}-character-long string. If you wish to have it '
-                          f'evaluated, please set the environment variable CHECKOV_RENDER_MAX_LEN '
-                          f'to {str(value_len + 1)} or to 0 to allow rendering of any length')
-        else:
-            if value_len < 5 or "." not in value:
-                # the shortest reference is 'var.a' and references are done via dot notation
-                return references_vertices
+        references_vertices = get_referenced_vertices_in_str_value(
+            str_value=value,
+            aliases=aliases,
+            resources_types=resources_types,
+        )
 
-            if cleanup_functions:
-                for func in cleanup_functions:
-                    value = func(value)
-            references_vertices = get_vertices_references(value, aliases, resources_types)
+    return references_vertices
+
+
+def get_referenced_vertices_in_str_value(
+    str_value: str,
+    aliases: dict[str, dict[str, str]],
+    resources_types: list[str],
+) -> list[TerraformVertexReference]:
+    references_vertices: "list[TerraformVertexReference]" = []
+
+    value_len = len(str_value)
+    if CHECKOV_RENDER_MAX_LEN and 0 < CHECKOV_RENDER_MAX_LEN < value_len:
+        logging.debug(
+            f'Rendering was skipped for a {value_len}-character-long string. If you wish to have it '
+            f'evaluated, please set the environment variable CHECKOV_RENDER_MAX_LEN '
+            f'to {str(value_len + 1)} or to 0 to allow rendering of any length'
+        )
+    else:
+        if value_len < 5 or "." not in str_value:
+            # the shortest reference is 'var.a' and references are done via dot notation
+            return references_vertices
+
+        str_value = remove_function_calls_from_str(str_value=str_value)
+        str_value = remove_index_pattern_from_str(str_value=str_value)
+        str_value = remove_interpolation(str_value=str_value)
+
+        references_vertices = get_vertices_references(str_value, aliases, resources_types)
 
     return references_vertices
 
@@ -244,7 +298,7 @@ def generate_possible_strings_from_wildcards(origin_string: str, max_entries: in
     return generated_strings
 
 
-def attribute_has_nested_attributes(attribute_key: str, attributes: Dict[str, Any]) -> bool:
+def attribute_has_nested_attributes(attribute_key: str, attributes: Dict[str, Any], attribute_is_leaf: Optional[Dict[str, bool]] = None) -> bool:
     """
     :param attribute_key: key inside the  `attributes` dictionary
     :param attributes:
@@ -252,8 +306,13 @@ def attribute_has_nested_attributes(attribute_key: str, attributes: Dict[str, An
     Example 1: if attributes.keys == [key1, key.key2], type(attributes[key1]) is dict and return True for key1
     Example 2: if attributes.keys == [key1, key1.0], type(attributes[key1]) is list and return True for key1
     """
-    prefixes_with_attribute_key = [a for a in attributes.keys() if a.startswith(attribute_key) and a != attribute_key]
-    if not any(re.findall(re.compile(r"\.\d+"), a) for a in prefixes_with_attribute_key):
+    if attribute_is_leaf is None:
+        attribute_is_leaf = {}
+    if attribute_is_leaf.get(attribute_key):
+        prefixes_with_attribute_key = []
+    else:
+        prefixes_with_attribute_key = [a for a in attributes if a.startswith(attribute_key) and a != attribute_key]
+    if not any(re.findall(NESTED_ATTRIBUTE_PATTERN, a) for a in prefixes_with_attribute_key):
         # if there aro no numeric parts in the key such as key1.0.key2
         return isinstance(attributes[attribute_key], dict)
     return isinstance(attributes[attribute_key], list) or isinstance(attributes[attribute_key], dict)
@@ -274,7 +333,7 @@ def attribute_has_dup_with_dynamic_attributes(attribute_key: str, attributes: di
         return False
 
 
-def get_related_resource_id(resource: dict[str, Any], file_path_to_referred_id: dict[str, str]) -> str:
+def get_related_resource_id(resource: dict[str, Any], file_path_to_referred_id: dict[str, str]) -> str | None:
     resource_id = resource.get(CustomAttributes.ID)
     # for external modules resources the id should start with the prefix module.[module_name]
     if resource.get(CustomAttributes.MODULE_DEPENDENCY):
@@ -286,9 +345,7 @@ def get_related_resource_id(resource: dict[str, Any], file_path_to_referred_id:
 
 def get_file_path_to_referred_id_networkx(graph_object: DiGraph) -> dict[str, str]:
     file_path_to_module_id = {}
-    for node in graph_object.nodes.values():
-        if node.get(CustomAttributes.BLOCK_TYPE) == BlockType.MODULE:
-            modules = node
+
     modules = [node for node in graph_object.nodes.values() if
                node.get(CustomAttributes.BLOCK_TYPE) == BlockType.MODULE]
     for modules_data in modules:
@@ -298,23 +355,61 @@ def get_file_path_to_referred_id_networkx(graph_object: DiGraph) -> dict[str, st
     return file_path_to_module_id
 
 
-def get_file_path_to_referred_id_igraph(graph_object: igraph.Graph) -> dict[str, str]:
+def get_file_path_to_referred_id_rustworkx(graph_object: DiGraph) -> dict[str, str]:
     file_path_to_module_id = {}
-    for v in graph_object.vs:
-        if v[CustomAttributes.BLOCK_TYPE] == BlockType.MODULE:
-            modules = v
-    modules = [v for v in graph_object.vs if
-               v[CustomAttributes.BLOCK_TYPE] == BlockType.MODULE]
-    for module_vertex in modules:
-        module_name = module_vertex['name']
-        module_content = module_vertex['attr'].get(CustomAttributes.CONFIG, {})
-        for path in module_content.get('batch', {}).get("__resolved__", []):
-            file_path_to_module_id[path] = f"module.{module_name}"
+
+    modules = [node for index, node in graph_object.nodes() if
+               node.get(CustomAttributes.BLOCK_TYPE) == BlockType.MODULE]
+    for modules_data in modules:
+        for module_name, module_content in modules_data.get(CustomAttributes.CONFIG, {}).items():
+            for path in module_content.get("__resolved__", []):
+                file_path_to_module_id[path] = f"module.{module_name}"
     return file_path_to_module_id
 
 
-def setup_file_path_to_referred_id(graph_object: DiGraph | igraph.Graph) -> dict[str, str]:
-    if isinstance(graph_object, igraph.Graph):
-        return get_file_path_to_referred_id_igraph(graph_object)
-    else:  # the default value of the graph framework is 'NETWORKX'
+def setup_file_path_to_referred_id(graph_object: LibraryGraph) -> dict[str, str]:
+    if isinstance(graph_object, DiGraph):
         return get_file_path_to_referred_id_networkx(graph_object)
+    else:
+        return get_file_path_to_referred_id_rustworkx(graph_object)
+
+
+def get_attribute_is_leaf(vertex: TerraformBlock) -> Dict[str, bool]:
+    attribute_is_leaf = {}
+    for attribute in vertex.attributes:
+        attribute_is_leaf[attribute] = True
+        other = '.'.join(attribute.split('.')[:-1])
+        if other in attribute_is_leaf:
+            attribute_is_leaf[other] = False
+    return attribute_is_leaf
+
+
+def join_double_quote_surrounded_dot_split(str_parts: list[str]) -> list[str]:
+    """Joins back split strings which enclosed a dot by double quotes
+
+    ex.
+
+    ['google_project_iam_binding', 'role["roles/logging', 'admin"]'] -> ['google_project_iam_binding', 'role["roles/logging.admin"]']
+
+    If someone finds a better solution feel free to replace it!
+    """
+
+    new_str_parts = []
+    joined_str_parts: list[str] = []
+    for part in str_parts:
+        if not joined_str_parts:
+            if '"' not in part:
+                new_str_parts.append(part)
+            elif part.count('"') >= 2:
+                new_str_parts.append(part)
+            else:
+                joined_str_parts.append(part)
+            continue
+
+        joined_str_parts.append(part)
+
+        if '"' in part:
+            new_str_parts.append(".".join(joined_str_parts))
+            joined_str_parts = []
+
+    return new_str_parts
